@@ -15,18 +15,21 @@ setClass("AllModelClass",
                         converge='numeric',
                         itemloc = 'numeric',
                         Prior='list',
+                        DIC='numeric',
                         BIC='numeric',
                         SABIC='numeric',
                         RMSEA='numeric',
                         null.mod = 'S4',
                         TLI = 'numeric',
                         logLik='numeric',
+                        logPrior='numeric',
                         SElogLik='numeric',
                         Call='call',
                         esttype='character',
                         nest='integer',
                         iter='numeric',
                         quadpts='numeric',
+                        theta_lim='numeric',
                         nfact='numeric',
                         prodlist='list',
                         constrain='list',
@@ -76,6 +79,7 @@ setClass("AllModelClass",
 #'     \item{\code{RMSEA}:}{Object of class \code{"numeric"}, root mean-square error of approximation for G2}
 #'     \item{\code{TLI}:}{Object of class \code{"numeric"}, Tucker-Lewis index for G2}
 #'     \item{\code{CFI}:}{Object of class \code{"numeric"}, CFI for G2}
+#'     \item{\code{logPrior}:}{Object of class \code{"numeric"}, log-prior distribution values}
 #'     \item{\code{logLik}:}{Object of class \code{"numeric"}, observed log-likelihood }
 #'     \item{\code{SElogLik}:}{Object of class \code{"numeric"}, Monte Carlo standard error for log-likelihood }
 #'     \item{\code{F}:}{Object of class \code{"matrix"}, unrotated factor loadings }
@@ -83,12 +87,10 @@ setClass("AllModelClass",
 #'     \item{\code{Theta}:}{Object of class \code{"matrix"}, ability grid }
 #'     \item{\code{Pl}:}{Object of class \code{"numeric"}, normed likelihoods for tabulated response}
 #'     \item{\code{prodlist}:}{Object of class \code{"list"}, list containing product combination of factors }
-#'     \item{\code{rotate}:}{Object of class \code{"character"}, type of rotation to be used in \code{summary}}
 #'     \item{\code{converge}:}{Object of class \code{"numeric"}, convergence diagnostic }
 #'     \item{\code{quadpts}:}{Object of class \code{"numeric"}, number of quadrature points }
 #'     \item{\code{esttype}:}{Object of class \code{"character"}, indicates whether estimation was 'EM' or 'MHRM'}
 #'     \item{\code{null.mod}:}{Object of class \code{"SingleGroupClass"}, null model}
-#'     \item{\code{Target}:}{Object of class \code{"numeric"}, dummy rotation matrix}
 #'     \item{\code{condnum}:}{Object of class \code{"numeric"}, condition number of information matrix}
 #'     \item{\code{secondordertest}:}{Object of class \code{"logical"}, indicate whether information matrix passes
 #'       second-order test}
@@ -123,8 +125,6 @@ setClass("AllModelClass",
 setClass(
     Class = 'SingleGroupClass', contains = 'AllModelClass',
     representation = representation(Pl='numeric',
-                                    Target='numeric',
-                                    rotate='character',
                                     bfactor='list',
                                     random='list'),
     validity = function(object) return(TRUE)
