@@ -16,13 +16,15 @@
 #'
 #' \dontrun{
 #' data(LSAT7)
+#' head(LSAT7)
 #' LSAT7full <- expand.table(LSAT7)
+#' head(LSAT7full)
 #'    }
 #'
 expand.table <- function(tabdata) {
     if(missing(tabdata)) missingMsg('tabdata')
     if (sum(tabdata[,ncol(tabdata)]) <= nrow(tabdata))
-        stop("Frequencies must be on the right of the data matrix.")
+        stop("Frequencies must be on the right of the data matrix.", call.=FALSE)
     itemnames <- colnames(tabdata[,1L:(ncol(tabdata) - 1L)])
     tabdata <- as.matrix(tabdata)
     fulldata <- c()
