@@ -44,8 +44,8 @@
 #'   numerically ordered data, with missing data coded as \code{NA}
 #' @param covdata a \code{data.frame} that consists of the \code{nrow(data)} by \code{K}
 #'   'person level' fixed and random predictors
-#' @param model an object returned from \code{mirt.model()} declaring how
-#'   the factor model is to be estimated. See \code{\link{mirt.model}} for
+#' @param model an object returned from, or a string to be passed to, \code{mirt.model()}
+#'   to declare how the IRT model is to be estimated. See \code{\link{mirt.model}} for
 #'   more details
 #' @param fixed a right sided R formula for specifying the fixed effect (aka 'explanatory')
 #'   predictors from \code{covdata} and \code{itemdesign}. To estimate the intercepts for
@@ -119,7 +119,7 @@
 #' mirtCluster()
 #'
 #' #specify IRT model
-#' model <- mirt.model('Theta = 1-10')
+#' model <- 'Theta = 1-10'
 #'
 #' #model with no person predictors
 #' mod0 <- mirt(data, model, itemtype = 'Rasch')
@@ -183,7 +183,7 @@
 #' data(SAT12)
 #' data <- key2binary(SAT12,
 #'                    key = c(1,4,5,2,3,1,2,1,3,1,2,4,2,1,5,3,4,4,1,4,3,3,4,1,3,5,1,3,1,5,4,5))
-#' model <- mirt.model('Theta = 1-32')
+#' model <- 'Theta = 1-32'
 #'
 #' # Suppose that the first 16 items were suspected to be easier than the last 16 items,
 #' #   and we wish to test this item structure hypothesis (more intercept designs are possible
@@ -278,8 +278,8 @@
 #' summary(mod1b)
 #'
 #' # specifying specific regression effects is accomplished by passing a list of formula
-#' model <- mirt.model('F1 = 1-5
-#'                      F2 = 6-10')
+#' model <- 'F1 = 1-5
+#'          F2 = 6-10'
 #' covdata$contvar <- rnorm(nrow(covdata))
 #' mod2 <- mirt(dat, model, itemtype = 'Rasch', covdata=covdata,
 #'         formula = list(F1 = ~ group + contvar, F2 = ~ group))
@@ -329,24 +329,25 @@ mixedmirt <- function(data, covdata = NULL, model, fixed = ~ 1, random = NULL, i
     itemdesignold <- if(is.null(itemdesign)) data.frame() else itemdesign
     if(length(itemtype) == 1L) itemtype <- rep(itemtype, ncol(data))
     if(any(itemtype %in% c('PC2PL', 'PC3PL', '2PLNRM', '3PLNRM', '3PLuNRM', '4PLNRM')))
-        stop('itemtype contains unsupported classes of items')
+        stop('itemtype contains unsupported classes of items', call.=FALSE)
     if(is(random, 'formula')) {
         random <- list(random)
     } else if(is.null(random)) random <- list()
     RETVALUES <- ifelse(is.character(pars), TRUE, FALSE)
-    if(!is.list(random)) stop('Incorrect input for random argument')
+    if(!is.list(random)) stop('Incorrect input for random argument', call.=FALSE)
     if(is.null(covdata)) covdata <- data.frame(UsElEsSvAR = factor(rep(1L, nrow(data))))
     if(is.null(itemdesign)){
         itemdesign <- data.frame(items = factor(1L:ncol(data)))
     } else {
         if(!is.null(itemdesign$items))
-            stop('itemdesign internally reserves the predictor name \'items\'. Please Change ')
+            stop('itemdesign internally reserves the predictor name \'items\'. Please Change ',
+                 call.=FALSE)
         itemdesign$items <- factor(1L:ncol(data))
     }
     if(!is.data.frame(covdata) || ! is.data.frame(itemdesign))
-        stop('Predictor variable inputs must be data.frame objects')
+        stop('Predictor variable inputs must be data.frame objects', call.=FALSE)
     if(nrow(covdata) != nrow(data))
-        stop('number of rows in covdata do not match number of rows in data')
+        stop('number of rows in covdata do not match number of rows in data', call.=FALSE)
     dropcases <- which(rowSums(is.na(covdata)) != 0)
     if(length(dropcases) > 0L){
         data <- data[-dropcases, ]
@@ -355,8 +356,8 @@ mixedmirt <- function(data, covdata = NULL, model, fixed = ~ 1, random = NULL, i
     pick <- sapply(covdata, is.numeric)
     if(any(pick)){
         if(any(covdata[,pick] > 10 || covdata[,pick] < -10))
-            warning('continuous variables in covdata should be rescaled to fall
-                     between -10 and 10 for better numerical stability')
+            warning('Continuous variables in covdata should be rescaled to fall
+                     between -10 and 10 for better numerical stability', call.=FALSE)
     }
     longdata <- reshape(data.frame(ID=1L:nrow(data), data, covdata), idvar='ID',
                         varying=list(1L:ncol(data) + 1L), direction='long')
@@ -369,7 +370,7 @@ mixedmirt <- function(data, covdata = NULL, model, fixed = ~ 1, random = NULL, i
     if(any(K > 2)){
         if(any(colnames(mm) %in% paste0('items', 1:ncol(data))))
             stop('fixed formulas do no support the \'items\' internal variable for
-                 polytomous items. Please remove')
+                 polytomous items. Please remove', call.=FALSE)
         mm <- mm[ , -1L, drop = FALSE]
     }
     if(return.design) return(list(X=mm, Z=NaN))
